/**
 * @file SimplePathFinder.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include "SimplePathFinder.h"

#include <limits>
#include <boost/foreach.hpp>

using std::numeric_limits;

namespace BPF
{
/**
 * Constructeur
 * @param[in] changeLineDuration Durée d'un changement de ligne, en seconde
 * @param[in] stationsFromLineMap Référence vers la map du BlasPathfinder
 * @param[in] crossingFromLineMap Référence vers la map du BlasPathfinder
 * @param[in] linesFromStationMap Référence vers la map du BlasPathfinder
 * @param[in] stationStationTimeInLine Référence vers la map du BlasPathfinder
 */
SinglePathFinder::SinglePathFinder(
	int changeLineDuration,
	const IntSetVect &stationsFromLineMap,
	const IntSetVect &crossingFromLineMap,
	const IntSetVect &linesFromStationMap,
	const TimeBetweenStationOnLineMap &stationStationTimeInLine):
		changeLineDuration_(changeLineDuration),
		stationsFromLineMap_(stationsFromLineMap),
		crossingFromLineMap_(crossingFromLineMap),
		linesFromStationMap_(linesFromStationMap),
		stationStationTimeInLine_(stationStationTimeInLine),
		lockedStations_(linesFromStationMap_.size(), false)
{
	currentWay_.reserve(linesFromStationMap_.size());
}

/**
 * Test tout les chemins possibles à partir du chemin actuel
 * et en continuant par la station demandée.
 */
void SinglePathFinder::testThisCrossing(
	int stationID     /**< [in] Station par laquelle le chemin se poursuit */,
	int destStationID /**< [in] Station de destination */,
	int lastLineID    /**< [in] Station précédente */,
	int currentTime   /**< [in] Temps écoulé depuis la station de départ */)
{
	if (currentTime >= bestTime_)
		return;

	if (lockedStations_[stationID])
		return;

	boost::unordered_map<int, int>::iterator minIter = minTimeForCrossingMap_.find(stationID);
	if (minIter != minTimeForCrossingMap_.end())
	{
		if (minIter->second <= (currentTime - changeLineDuration_))
			return;
		else
			minIter->second = currentTime;
	}
	else
		minTimeForCrossingMap_.insert(std::make_pair(stationID, currentTime));

	std::pair<int, int> stationLineKey(stationID, lastLineID);
	MinTimeForCrossingLineMapType::iterator minLineIter =
		minTimeForCrossingLineMap_.find(stationLineKey);
	if (minLineIter != minTimeForCrossingLineMap_.end())
	{
		if (minLineIter->second <= currentTime)
			return;
		else
			minLineIter->second = currentTime;
	}
	else
		minTimeForCrossingLineMap_.insert(std::make_pair(stationLineKey, currentTime));

	int newTime = currentTime;
	if (lastLineID != -1)
		newTime += changeLineDuration_;

	lockedStations_[stationID] = true;

	const IntSet &linesOfStationSet = linesFromStationMap_[stationID];
	BOOST_FOREACH(int lineID, linesOfStationSet)
		testThisLine(
			lineID,
			destStationID,
			stationID,
			newTime);

	lockedStations_[stationID] = false;
}

/**
 * Test tout les chemins possible à partir du chemin actuel
 * et en continuant sur la ligne demandée.
 */
void SinglePathFinder::testThisLine(
	int lineID        /**< [in] Ligne sur laquelle le chemin se poursuit */,
	int destStationID /**< [in] Station de destination */,
	int lastStationID /**< [in] Station précédente */,
	int currentTime   /**< [in] Temps écoulé depuis la station de départ */)
{
	if (currentTime >= bestTime_)
		return;

	if (linesFromStationMap_[destStationID].count(lineID))
	{
		LineStationToStationKey key(lineID, lastStationID, destStationID);
		TimeBetweenStationOnLineMap::const_iterator stepDuration = stationStationTimeInLine_.find(key);
		assert(stepDuration != stationStationTimeInLine_.end());
		int newTime = currentTime + stepDuration->second;
		if (newTime < bestTime_)
		{
			bestTime_ = newTime;
			bestWay_ = currentWay_;
			bestWay_.push_back(Step(lineID, destStationID, stepDuration->second));
		}
	}

	const IntSet &stationsOfLineSet = crossingFromLineMap_[lineID];
	BOOST_FOREACH(int stationID, stationsOfLineSet)
	{
		if (stationID == lastStationID)
			continue;

		LineStationToStationKey key(lineID, lastStationID, stationID);

		TimeBetweenStationOnLineMap::const_iterator stepDuration = stationStationTimeInLine_.find(key);
		int newTime = currentTime + stepDuration->second;

		currentWay_.push_back(Step(lineID, stationID, stepDuration->second));

		testThisCrossing(
			stationID,
			destStationID,
			lineID,
			newTime);

		currentWay_.pop_back();
	}
}

/**
 * Recherche le chemin le plus rapide pour aller d'une station à une autre
 * (Si les stations de début et fin sont identiques, le chemin sera vide)
 */
void SinglePathFinder::computeBestWay(
	int stationID              /**< [in] Station de départ */,
	int destStationID          /**< [in] Station d'arrivée */,
	std::vector<Step> &bestWay /**< [out] Meilleur chemin trouvé */,
	int &bestTime              /**< [out] Meilleur temps trouvé */)
{
	bestWay.clear();

	if (stationID == destStationID)
	{
		bestTime = 0;
		return;
	}

	bestTime_ = numeric_limits<int>::max();

	testThisCrossing(
		stationID,
		destStationID,
		-1,
		0);

	bestWay = bestWay_;
	bestTime = bestTime_;
}

}
