/**
 * @file SimplePathFinder.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef SINGLEPATHFINDER_H_INCLUDED
#define SINGLEPATHFINDER_H_INCLUDED

#include <vector>
#include <boost/unordered_map.hpp>
#include <boost/unordered_set.hpp>
#include <boost/noncopyable.hpp>
#include "types.h"

namespace BPF
{

/**
 *  @brief  Recherche plus rapide chemin
 *
 *  Classe qui recherche le chemin le plus rapide d'une station à une autre
 * sans station intermédiaire
*/
class SinglePathFinder : boost::noncopyable
{
	typedef boost::unordered_map<std::pair<int, int> , int> MinTimeForCrossingLineMapType;

  //! Temps minimum pour arriver à une station, à partir de la station de départ
	boost::unordered_map<int, int> minTimeForCrossingMap_;

  /** Temps minimum pour arriver à une station,
   *  en arrivant d'une ligne définie, à partir de la station de départ */
	MinTimeForCrossingLineMapType minTimeForCrossingLineMap_;

	//!Durée d'une correspondance
	int changeLineDuration_;

	//!Référence vers stationsFromLineMap_ dans le BlasPathFinder
	const IntSetVect &stationsFromLineMap_;

	//!Référence vers crossingFromLineMap_ dans le BlasPathFinder
	const IntSetVect &crossingFromLineMap_;

	//!Référence vers linesFromStationMap_ dans le BlasPathFinder
	const IntSetVect &linesFromStationMap_;

	//!Référence vers stationStationTimeInLine_ dans le BlasPathFinder
	const TimeBetweenStationOnLineMap &stationStationTimeInLine_;

	//!Sait si une station est déjà intégrée dans le chemin en cours
	std::vector<bool> lockedStations_;

	//!Contient le chemin en cours
	std::vector<Step> currentWay_;

	//!Contient le meilleur chemin connu
	std::vector<Step> bestWay_;

	//!Contient le meilleur temps connu
	int bestTime_;

  //!Test tout les chemins possibles à partir du chemin actuel et en continuant sur cette station
	void testThisCrossing(
		int stationID,
		int destStationID,
		int lastLineID,
		int currentTime);

  //!Test tout les chemins possibles à partir du chemin actuel et en continuant sur cette ligne
	void testThisLine(
		int lineID,
		int destStationID,
		int lastStationID,
		int currentTime);

public:
	SinglePathFinder(
		int changeLineDuration,
		const IntSetVect &stationsFromLineMap,
		const IntSetVect &crossingFromLineMap,
		const IntSetVect &linesFromStationMap,
		const TimeBetweenStationOnLineMap &stationStationTimeInLine
	);

	//!Calcule le chemin le plus rapide d'une station à une autre, sans intermédiaire
	void computeBestWay(
		int stationID,
		int destStationID,
		std::vector<Step> &bestWay,
		int &bestTime);
};

}


#endif // SINGLEPATHFINDER_H_INCLUDED
