/**
 * @file DVPCollate.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include "DVPCollate.h"

#include <map>
#include <string>
#include <iostream>

using std::map;
using std::wstring;

namespace
{

/**
 * Transformation de caractère majuscule ou accentué
 * en caractère minuscule simple.
 */
class LatinCharTransform
{
  //! Table de conversion
	std::map<wchar_t, wstring> toBasicCharMap_;

public:
  //! Initialisation de la table de conversion
	LatinCharTransform()
	{
		for (wchar_t charIter = L'A'; charIter <= L'Z'; ++charIter)
			toBasicCharMap_[charIter] = wstring(1, charIter + (L'a' - L'A'));
		for (wchar_t charIter = L'\u00C0'; charIter <= L'\u00C5'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"a");
		toBasicCharMap_[L'\u00C6'] = wstring(L"ae");
		toBasicCharMap_[L'\u00C7'] = wstring(L"c");
		for (wchar_t charIter = L'\u00C8'; charIter <= L'\u00CB'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"e");
		for (wchar_t charIter = L'\u00CC'; charIter <= L'\u00CF'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"i");
		for (wchar_t charIter = L'\u00D2'; charIter <= L'\u00D6'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"o");
		for (wchar_t charIter = L'\u00D9'; charIter <= L'\u00DC'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"u");
		toBasicCharMap_[L'\u00DD'] = wstring(L"y");

		for (wchar_t charIter = L'\u00E0'; charIter <= L'\u00E5'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"a");
		toBasicCharMap_[L'\u00E6'] = wstring(L"ae");
		for (wchar_t charIter = L'\u00E8'; charIter <= L'\u00EB'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"e");
		for (wchar_t charIter = L'\u00EC'; charIter <= L'\u00EF'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"i");
		for (wchar_t charIter = L'\u00F2'; charIter <= L'\u00F6'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"o");
		for (wchar_t charIter = L'\u00F9'; charIter <= L'\u00FC'; ++charIter)
			toBasicCharMap_[charIter] = wstring(L"u");
		toBasicCharMap_[L'\u00FD'] = wstring(L"y");
		toBasicCharMap_[L'\u00FF'] = wstring(L"y");
		toBasicCharMap_[L'\u00E7'] = wstring(L"c");
	}

  /**
   * Transforme un caractère wchar_t en une chaîne équivalente
   * sans accent ni majuscule.
   * La valeur de retour est une chaîne car par example 'æ' donnera "ae"
   */
	const wstring& transform(wchar_t c)
	{
		std::map<wchar_t, wstring>::const_iterator iter =
			toBasicCharMap_.find(c);
		if (iter == toBasicCharMap_.end())
		{
			toBasicCharMap_[c] = wstring(1, c);
			return toBasicCharMap_[c];
		}

		return iter->second;
	}
};

LatinCharTransform char_transform_instance;

}

template<typename _CharT>
int DVPCollate<_CharT>::do_compare(
	const _CharT* __lo1,
	const _CharT* __hi1,
	const _CharT* __lo2,
	const _CharT* __hi2) const
{
	string_t str1;
	string_t str2;

	//Conversion de la chaîne 1, sans accent ni majuscule
	for (/*rien*/; __lo1 < __hi1; ++__lo1)
	{
	  string_t charStr1 = char_transform_instance.transform(*__lo1);
		str1.insert(str1.end(), charStr1.begin(), charStr1.end());
	}
	//Conversion de la chaîne 2, sans accent ni majuscule
	for (/*rien*/; __lo2 < __hi2; ++__lo2)
	{
	  string_t charStr2 = char_transform_instance.transform(*__lo2);
		str2.insert(str2.end(), charStr2.begin(), charStr2.end());
	}

  //Comparaison des deux chaînes en utilisant le collate standard
	return std::collate<_CharT>::do_compare(
					 &str1[0],
					 &str1[0] + str1.length(),
					 &str2[0],
					 &str2[0] + str2.length());
}

template<typename _CharT>
std::basic_string<_CharT> DVPCollate<_CharT>::do_transform(
	const _CharT* __lo,
	const _CharT* __hi) const
{
	string_t res;
	for (/*rien*/; __lo != __hi; ++__lo)
	{
	  const string_t& charStr = char_transform_instance.transform(*__lo);
		res.insert(res.end(), charStr.begin(), charStr.end());
	}
	return res;
}

template<typename _CharT>
long DVPCollate<_CharT>::do_hash(
	const _CharT* __lo,
	const _CharT* __hi) const
{
  std::basic_string<_CharT> res = do_transform(__lo, __hi);
  return std::collate<_CharT>::do_hash(&res[0], &res[0] + res.length());
}

//Pour le moment, seule la version wchar_t existe
template class DVPCollate<wchar_t>;
