/**
 * @file Engine.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include "Engine.h"

#include <iostream>
#include <boost/foreach.hpp>
#include "PathResult.h"
#include "Line.h"
#include "Station.h"
#include "IStationDBLoader.h"
#include "IPathFinder.h"

using std::wcin;
using std::endl;
using std::string;
using std::wstring;
using std::vector;
using std::list;
using boost::shared_ptr;

Engine::Engine(
  boost::shared_ptr<IStationDBLoader> dbLoader,
  boost::shared_ptr<IPathFinder> pathFinder
):
	dbLoader_(dbLoader),
	pathFinder_(pathFinder),
	pathOptimisation_(false),
	stepDuration_(-1),
	changeDuration_(-1)
{

}

void Engine::userSetStepDuration(int stepDuration)
{
	stepDuration_ = stepDuration;
}

void Engine::userSetChangeDuration(int changeDuration)
{
	changeDuration_ = changeDuration;
}

void Engine::userSetStartStation(const std::wstring &startStation)
{
	startStationName_ = startStation;
}

void Engine::userSetEndStation(const std::wstring &endStation)
{
	endStationName_ = endStation;
}

void Engine::userAddIntermediateStations(const std::vector<std::wstring> &stepStationVect)
{
	stationNameStepList_.insert(stationNameStepList_.end(), stepStationVect.begin(), stepStationVect.end());
}

void Engine::userWantPathOptimisation()
{
	pathOptimisation_ = true;
}

/**
 * run() va charger la base de donnée,
 * demander à l'utilisateur les paramètres requis pas encore définis,
 * lancer le calcul du meilleur chemin,
 * et enfin l'afficher.
 */
void Engine::run()
{
  try
  {
    dbLoader_->load(stationDB_);
  }
  catch(IStationDBLoader::LoadError &ex)
  {
    display_.dataBaseLoadError(ex.getMessage());
    return;
  }

	StationHandler &stationHandler = stationDB_.getStationHandler();

	while(stepDuration_ < 0)
		display_.queryStepDuration(stepDuration_);
	while(changeDuration_ < 0)
		display_.queryChangeDuration(changeDuration_);
	while(stationHandler.getStationAware(startStationName_) == false)
		display_.queryStartStation(startStationName_);
	while(stationHandler.getStationAware(endStationName_) == false)
		display_.queryEndStation(endStationName_);

	pathFinder_->init(stationDB_, stepDuration_, changeDuration_);

	SharedConstStation startStation = stationHandler.getStationAware(startStationName_);
	SharedConstStation endStation = stationHandler.getStationAware(endStationName_);
	vector<SharedConstStation> stationStepList;
	stationStepList.clear();

	BOOST_FOREACH(wstring &stationName, stationNameStepList_)
	{
		SharedConstStation station = stationHandler.getStationAware(stationName);
		while(station == NULL)
		{
      display_.queryIntermediateStation(stationName);
      station = stationHandler.getStationAware(stationName);
		}
		stationStepList.push_back(station);
	}

	shared_ptr<PathResult> pathResult = pathFinder_->find(
    stationDB_,
		startStation,
		stationStepList,
		endStation,
		pathOptimisation_);

	display_.displayResult(*pathResult);
}

