/**
 * @file IPathFinder.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef IPATHFINDER_H_INCLUDED
#define IPATHFINDER_H_INCLUDED

#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>

class StationsDB;
class PathResult;
class Station;
typedef boost::shared_ptr<const Station> SharedConstStation;

//! Interface des class de recherche de chemin
/**
 * Interface pour les class qui doivent chercher le chemin le plus court
 */
class IPathFinder : boost::noncopyable
{
public:
  virtual ~IPathFinder() {} /**< Virtual pour permettre l'héritage */

  /**
   * Initialise le contenu du PathFinder à partir des données qui lui sont passées.
   * Si ces données changent, cette fonction doit pouvoir être rappelée.
   * @param[in] stationDB          Base de donnée des stations
   * @param[in] stepDuration       Durée entre deux stations, en secondes
   * @param[in] changeLineDuration Durée d'une correspondance, en secondes
   */
  virtual void init(
		const StationsDB &stationDB,
		int stepDuration,
		int changeLineDuration) = 0;

  /**
   * Recherche le chemin le plus rapide.
   * Cette fonction peut être appelée plusieurs fois.
   * @param[in] stationDB     Base de donnée des stations
   * @param[in] startStation  Station de départ
   * @param[in] interStations Liste des stations intermédiaires
   * @param[in] endStation    Station d'arrivée
   * @param[in] isOptim       True si l'ordre des stations intermédiaires peut être optimisé
   * @return Meilleur chemin trouvé
   */
  virtual boost::shared_ptr<PathResult> find(
    const StationsDB &stationDB,
    SharedConstStation startStation,
    const std::vector<SharedConstStation> &interStations,
    SharedConstStation endStation,
    bool isOptim) const = 0;
};

#endif // IPATHFINDER_H_INCLUDED
