/**
 * @file Line.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef LINE_H_INCLUDED
#define LINE_H_INCLUDED

#include <string>
#include <vector>
#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>

class Station;
typedef boost::shared_ptr<const Station> SharedConstStation;

//! Contient les données d'une ligne de métro
class Line : boost::noncopyable
{
  //! Nom de la ligne
	const std::wstring name_;
	//! Identifiant unique de la ligne
	const int id_;
	//! Liste des stations de la ligne
	std::vector<SharedConstStation> stationVect_;

	Line(
	  int id /**< [in] Identifiant unique */,
    const std::wstring &name /**< [in] Nom de la ligne */);


public:
	typedef std::vector<SharedConstStation>::iterator iterator;
	typedef std::vector<SharedConstStation>::const_iterator const_iterator;

	static boost::shared_ptr<Line> Create(
    int id /**< [in] Identifiant unique */,
    const std::wstring &name /**< [in] Nom de la ligne */);

	const std::wstring& getName() const;

	int getID() const;

	void addStation(
    SharedConstStation newStation /**< [in] Station à ajouter à la ligne*/);

  //! Renvoi la station de fin, dans une direction donnée
  /**
   * Renvoi la station de fin, dans la direction définie
   * par une station de départ et une station d'arrivée.
   * Les deux stations doivent être différentes et non null.
   * @param[in] X Station de départ (ne doit pas être null)
   * @param[in] Y Station d'arrivée (ne doit pas être null)
   * @return Station du terminus de la ligne, dans le sens donné
   */
  SharedConstStation getEndStationInThisDirection(
    SharedConstStation X,
    SharedConstStation Y) const;

	const_iterator begin() const;
	const_iterator end() const;
};

typedef boost::shared_ptr<Line> SharedLine;
typedef boost::shared_ptr<const Line> SharedConstLine;

inline const std::wstring& Line::getName() const
{
	return name_;
}

inline int Line::getID() const
{
	return id_;
}

inline Line::const_iterator Line::begin() const
{
	return stationVect_.begin();
}

inline Line::const_iterator Line::end() const
{
	return stationVect_.end();
}

#endif // LINE_H_INCLUDED
