/**
 * @file NoganePathFinder.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include "NoganePathFinder.h"

#include <iostream>
#include <boost/functional/hash.hpp>
#include <boost/foreach.hpp>
#include <set>
#include <ctime>
#include "StationsDB.h"
#include "PathResult.h"
#include "Line.h"
#include "Station.h"
#include "BPF/SimplePathFinder.h"
#include "BPF/stat_tools.h"

using std::endl;
using std::vector;
using std::pair;
using std::make_pair;
using std::numeric_limits;
using std::swap;
using boost::unordered_map;
using boost::shared_ptr;

using BPF::IntSet;
using BPF::LineStationToStationKey;
using BPF::IntSetVect;
using BPF::IntSetConstIterator;
using BPF::Step;

namespace
{
/**
 * Calcule la durée d'un trajet, en seconde.
 * @param[in] changeLineDuration durée d'un changement de ligne
 *    (temps perdu à une station intermédiaire même si on reste sur la même ligne)
 * @param[in] interToInterTime Map donnant la durée minimum trouvée
 *    pour deux stations données.
 * @param[in] stepList Liste des stations parcourues.(début + intermédiaires + fin)
 * @return Temps calculé, en seconde
 */
int computeIntListTime(
	int changeLineDuration,
	const unordered_map<pair<int, int>, int> &interToInterTime,
	const vector<int> stepList)
{

	typedef vector<int>::const_iterator StationListIterator;
	StationListIterator stationIter2 = stepList.begin();
	stationIter2++;

	int duration = 0;
	for (StationListIterator stationIter1 = stepList.begin();
			 stationIter2 != stepList.end();
			 ++stationIter1, ++stationIter2)
	{
		unordered_map<pair<int, int>, int>::const_iterator timeIter =
			interToInterTime.find(make_pair(
															*stationIter1,
															*stationIter2));
		assert(timeIter != interToInterTime.end());
		int temp = timeIter->second;
		duration += temp;
		duration += changeLineDuration;
	}
	duration -= changeLineDuration;
	return duration;
}

}

NoganePathFinder::NoganePathFinder():
		stepDuration_(0),
		changeLineDuration_(0)
{
}

void NoganePathFinder::init(
	const StationsDB &stationDB,
	int stepDuration,
	int changeLineDuration)
{
	stepDuration_ = stepDuration;
	changeLineDuration_ = changeLineDuration;

	stationStationTimeInLine_.clear();
	stationsFromLineMap_.clear();
	crossingFromLineMap_.clear();
	linesFromStationMap_.clear();

	const LineHandler &lineHandler = stationDB.getLineHandler();
	const StationHandler &stationHandler = stationDB.getStationHandler();
	linesFromStationMap_.assign(stationHandler.size(), IntSet());
	stationsFromLineMap_.assign(lineHandler.size(), IntSet());
	crossingFromLineMap_.assign(lineHandler.size(), IntSet());

	//Parcours de toute les stations de toutes les lignes, pour remplir
	//linesFromStationMap_, stationsFromLineMap_, et stationStationTimeInLine_
	typedef std::pair<std::wstring ,SharedConstLine> PairIntLine;
	BOOST_FOREACH(const PairIntLine &lineIter, lineHandler)
	{
		int stationNum1 = 0;
		SharedConstLine line = lineIter.second;
		for (Line::const_iterator stationIter = line->begin();
				 stationIter != line->end();
				 stationIter++, stationNum1++)
		{
			SharedConstStation station = *stationIter;
			linesFromStationMap_[station->getID()].insert(line->getID());
			stationsFromLineMap_[line->getID()].insert(station->getID());

			int stationNum2 = 0;
			for (Line::const_iterator stationIter2 = line->begin();
					 stationIter2 != line->end();
					 stationIter2++, stationNum2++)
			{
				SharedConstStation station2 = *stationIter2;
				int duration = abs(stationNum2 - stationNum1) * stepDuration;
				LineStationToStationKey key1(line->getID(), station->getID(), station2->getID());
				LineStationToStationKey key2(line->getID(), station2->getID(), station->getID());
				stationStationTimeInLine_[key1] = duration;
				stationStationTimeInLine_[key2] = duration;
			}
		}
	}

	//Parcours la stationsFromLineMap_ pour remplir la crossingFromLineMap_
	//stationsFromLineMap_ est un vecteur qui prend en clef le numéro de ligne
	//donc lineID représentera bien le numéro de ligne
	int lineID = 0;
	for (IntSetVect::const_iterator stationsOfLineIter = stationsFromLineMap_.begin();
			 stationsOfLineIter != stationsFromLineMap_.end();
			 ++stationsOfLineIter, ++lineID)
	{
		const IntSet &stationsOfLineSet = *stationsOfLineIter;

		BOOST_FOREACH(const int stationID, stationsOfLineSet)
		{
			if (linesFromStationMap_[stationID].size() > 1)
        crossingFromLineMap_[lineID].insert(stationID);
		}
	}
}

void NoganePathFinder::findBestPathForTwoStation(
	int stationID,
	int destID,
	int &bestTime,
	std::vector<Step> &bestWay) const
{
	//C'est juste une astuce pour quand changeLineDuration_
	//et stepDuration_ sont à zéro. Ca limite la complexité du résultat.
	int changeLineDuration = changeLineDuration_;
	if((changeLineDuration_ * stepDuration_) == 0)
    changeLineDuration = 1;

	BPF::SinglePathFinder finder(
		changeLineDuration,
		stationsFromLineMap_,
		crossingFromLineMap_,
		linesFromStationMap_,
		stationStationTimeInLine_);

	finder.computeBestWay(stationID, destID, bestWay, bestTime);
}

void NoganePathFinder::findBestPathForStationList(
	const std::vector<int> &stationList,
	int &bestTime,
	std::vector<Step> &bestWay) const
{
	assert(stationList.size() > 1);

	vector<Step> finalBestWay;
	int finalBestTime = 0;
	finalBestWay.reserve(100);

	vector<int>::const_iterator stationIter2 = stationList.begin();
	stationIter2++;
	for (vector<int>::const_iterator stationIter = stationList.begin();
			 stationIter2 != stationList.end();
			 stationIter++, stationIter2++)
	{
		int station1 = *stationIter;
		int station2 = *stationIter2;
		if (station1 == station2)
			continue;
		findBestPathForTwoStation(station1, station2, bestTime, bestWay);
		if (bestTime == numeric_limits<int>::max())
		{
			finalBestWay.clear();
			return;
		}
		finalBestWay.insert(finalBestWay.end(), bestWay.begin(), bestWay.end());
		finalBestTime += bestTime;
		finalBestTime += changeLineDuration_;
	}

	bestWay = finalBestWay;
	bestTime = finalBestTime - changeLineDuration_;
}

boost::shared_ptr<PathResult> NoganePathFinder::find(
	const StationsDB &stationDB,
	SharedConstStation X,
	const std::vector<SharedConstStation> &stepList,
	SharedConstStation Y,
	bool optimizeStep) const
{
	const StationHandler &stationHandler = stationDB.getStationHandler();
	const LineHandler &lineHandler = stationDB.getLineHandler();

	vector<int> stepListInt;
	BOOST_FOREACH(const SharedConstStation &station, stepList)
		stepListInt.push_back(station->getID());

	shared_ptr<PathResult> result(new PathResult);

	int bestTime = numeric_limits<int>::max();
	vector<Step> bestWay;

	if (optimizeStep == false)
	{
		stepListInt.push_back(Y->getID());
		stepListInt.insert(stepListInt.begin(), X->getID());

		findBestPathForStationList(stepListInt, bestTime, bestWay);
		if (bestTime == numeric_limits<int>::max())
			return result;
	}
	else
	{
	  //Suppression des doublons
	  stepListInt.erase(unique(stepListInt.begin(), stepListInt.end()), stepListInt.end());
	  stepListInt.erase(remove(stepListInt.begin(), stepListInt.end(), X->getID()), stepListInt.end());
	  stepListInt.erase(remove(stepListInt.begin(), stepListInt.end(), Y->getID()), stepListInt.end());

		///////////////////////////////////////////////////////////////////////////////////////
		//Calcul des temps entre toutes les stations concernées
		///////////////////////////////////////////////////////////////////////////////////////
		unordered_map<pair<int, int>, int> stationToStationTime;
		BOOST_FOREACH(int station2ID, stepListInt)
		{
			int station1ID = X->getID();
			int station3ID = Y->getID();

      //Station de départ vers toutes les stations intermédiaires
			findBestPathForTwoStation(station1ID, station2ID, bestTime, bestWay);
			if (bestTime == numeric_limits<int>::max())
				return result;
			stationToStationTime.insert(make_pair(make_pair(station1ID, station2ID), bestTime));
			stationToStationTime.insert(make_pair(make_pair(station2ID, station1ID), bestTime));

      //Toutes les stations intermédiaires vers station de fin
			findBestPathForTwoStation(station2ID, station3ID, bestTime, bestWay);
			if (bestTime == numeric_limits<int>::max())
				return result;
			stationToStationTime.insert(make_pair(make_pair(station2ID, station3ID), bestTime));
			stationToStationTime.insert(make_pair(make_pair(station3ID, station2ID), bestTime));
		}

		//Toutes les stations intermédiaires entre elles.
		//(Si A->B a été calculé, on ne calcule pas B<-A)
		typedef vector<SharedConstStation>::const_iterator StepListIterator;
		for (StepListIterator stepIter1 = stepList.begin();
				 stepIter1 != stepList.end();
				 ++stepIter1)
		{
			for(StepListIterator stepIter2 = stepIter1 + 1;
          stepIter2 != stepList.end();
          ++stepIter2)
			{
				int station1ID = (*stepIter1)->getID();
				int station2ID = (*stepIter2)->getID();
				findBestPathForTwoStation(station1ID, station2ID, bestTime, bestWay);
				if (bestTime == numeric_limits<int>::max())
					return result;
				stationToStationTime.insert(make_pair(make_pair(station1ID, station2ID), bestTime));
				stationToStationTime.insert(make_pair(make_pair(station2ID, station1ID), bestTime));
			}
		}
		///////////////////////////////////////////////////////////////////////////////////////
		//FIN Calcul des temps entre toutes les stations concernées
		///////////////////////////////////////////////////////////////////////////////////////

		//Si il n'y as pas trop d'étapes,
		//on peut tester toutes les combinaisons
		if (stepListInt.size() < 11)
		{
			vector<vector<int> > allOrderList;

			BPF::allCombin(stepListInt, allOrderList);
			vector<int> bestBestWay;
			bestTime = numeric_limits<int>::max();

			BOOST_FOREACH(const vector<int> &orderedList, allOrderList)
			{
				vector<int> stationList;
				stationList.reserve(orderedList.size() + 2);
				stationList.push_back(X->getID());
				stationList.insert(stationList.end(), orderedList.begin(), orderedList.end());
				stationList.push_back(Y->getID());

				int duration = computeIntListTime(changeLineDuration_, stationToStationTime, stationList);
				if (duration < bestTime)
				{
					bestTime = duration;
					bestBestWay = stationList;
				}
			}

			findBestPathForStationList(bestBestWay, bestTime, bestWay);
		}
		//Sinon, on utilise une autre méthode
		//qui ne certifie pas de trouver la meilleure solution
		else
		{
			srand(static_cast<unsigned int>(time(0)));
			int bestBestTime = numeric_limits<int>::max();
			stepListInt.push_back(Y->getID());
			stepListInt.insert(stepListInt.begin(), X->getID());
			unsigned int idx1 = 1;
			unsigned int idx2 = 2;
			for (int i = 0; i < 1000; ++i)
			{
				//int idx1 = (rand() % (stepListInt.size() - 2)) + 1;
				//int idx2 = (rand() % (stepListInt.size() - 2)) + 1;

				swap(stepListInt[idx1], stepListInt[idx2]);

				int currentListTime = computeIntListTime(changeLineDuration_, stationToStationTime, stepListInt);
				if (currentListTime < bestBestTime)
					bestBestTime = currentListTime;
				else
					swap(stepListInt[idx1], stepListInt[idx2]);

				idx2++;
				if (idx2 >= (stepListInt.size() - 1))
				{
					idx2 = 1;

				  idx1++;
				  if (idx1 >= (stepListInt.size() - 1))
				  {
				    idx1 = 1;
				    idx2 = 2;
				  }
				}
			}
			findBestPathForStationList(stepListInt, bestTime, bestWay);
		}
	}

	if (bestTime == numeric_limits<int>::max())
		return result;

	//////////////////////////////////////////////////////////////////////
	//             Génération de la liste résultante
	//////////////////////////////////////////////////////////////////////
	SharedConstStation lastStation = X;
	int changeDuration = 0;
	BOOST_FOREACH(Step &stepIter, bestWay)
	{
		SharedConstLine currentLine = lineHandler.getLine(stepIter.lineID);

		result->addStep(
			PathResult::Step(
				lastStation,
				stationHandler.getStation(stepIter.statID),
				currentLine,
				changeDuration,
				stepIter.duration));

		lastStation = stationHandler.getStation(stepIter.statID);
		changeDuration = changeLineDuration_;
	}

	if(bestWay.empty())
	{
		result->addStep(
			PathResult::Step(
				X,
				X,
				SharedConstLine(),
				0,
				0));
	}

	return result;
}
