/**
 * @file StationHandler.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include "StationHandler.h"

#include <iostream>
#include <boost/foreach.hpp>
#include "Station.h"
#include "DVPCollate.h"

using std::make_pair;
using std::wstring;

int StationHandler::NextID = 0;

void StationHandler::addStation(SharedConstStation newStation)
{
	assert(newStation != false);
	assert(stationIntMap_.find(newStation->getID()) == stationIntMap_.end());
	stationIntMap_.insert(make_pair(newStation->getID(), newStation));
	stationMap_.insert(make_pair(newStation->getName(), newStation));
	stationVect_.push_back(newStation);
}


SharedConstStation StationHandler::createStation(const std::wstring stationName)
{
	SharedConstStation newStation = Station::Create(NextID, stationName);
	NextID++;
	addStation(newStation);
	return newStation;
}

SharedConstStation StationHandler::getStation(const std::wstring &name) const
{
	StationMapType::const_iterator stationIter = stationMap_.find(name);
	if (stationIter == stationMap_.end())
		return SharedConstStation();
	else
		return stationIter->second;
}

/**
 * Le problème avec un collate qui supprime les accents et majuscules,
 * c'est que si deux noms de stations ne diffèrent que par un accent,
 * l'une des deux ne sera plus accessible. Donc la recherche est d'abord faite par
 * getStation, et si on ne trouve pas, en dernier recours, on utilise DVPCollate.
 *
 * Il aurait été possible de faire une autre map spécialement pour getStationAware.
 * Cela aurait évité le foreach, mais pas très utile pour le moment.
 */
SharedConstStation StationHandler::getStationAware(const std::wstring &name) const
{
	StationMapType::const_iterator stationIter = stationMap_.find(name);
	if (stationIter == stationMap_.end())
	{
		const DVPCollate<wchar_t> dvpcollate;
		wstring str1 = dvpcollate.transform(name.c_str(), name.c_str() + name.length());

		typedef std::pair<std::wstring, SharedConstStation> PairWstringStation;
		BOOST_FOREACH(const PairWstringStation &stationIter, stationMap_)
		{
			const wstring &iterName = stationIter.first;
			wstring str2 = dvpcollate.transform(iterName.c_str(), iterName.c_str() + iterName.length());
			if(str1 == str2)
				return stationIter.second;
		}
		return SharedConstStation();
	}
	else
		return stationIter->second;
}


SharedConstStation StationHandler::getStation(const int id) const
{
	StationIntMapType::const_iterator stationIter = stationIntMap_.find(id);
	if (stationIter == stationIntMap_.end())
		return SharedConstStation();
	else
		return stationIter->second;
}

void StationHandler::clear()
{
	stationMap_.clear();
	stationIntMap_.clear();
	stationVect_.clear();
	NextID = 0;
}

