/**
 * @file StationHandler.h
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#ifndef STATIONHANDLER_H_INCLUDED
#define STATIONHANDLER_H_INCLUDED

#include <map>
#include <vector>
#include <string>
#include <stdexcept>
#include <boost/shared_ptr.hpp>
#include <boost/noncopyable.hpp>

class Station;
typedef boost::shared_ptr<const Station> SharedConstStation;

//! Gestionnaire de stations
/**
 * Contient toutes les stations, et permet de les retrouver facilement
 * @see Station
 */
class StationHandler : boost::noncopyable
{
	typedef std::map<std::wstring, SharedConstStation> StationMapType;
	typedef std::map<int, SharedConstStation> StationIntMapType;
	typedef std::vector<SharedConstStation> StationVectorType;
public:
  typedef StationVectorType::iterator iterator;
  typedef StationVectorType::const_iterator const_iterator;
private:
  static int NextID; //!< ID de la prochaine station qui sera créée. (auto_increment)

  StationMapType stationMap_; //!< Pour trouver une station à partir de son nom
  StationIntMapType stationIntMap_; //!< Pour trouver une station à partir de son ID
  StationVectorType stationVect_; //!< Juste pour contenir les stations

  void addStation(SharedConstStation newStation);

public:
  SharedConstStation createStation(const std::wstring stationName);

  //! Trouve une station par son nom, sans tolérer d'erreur.
  SharedConstStation getStation(const std::wstring &name) const;

  //! Trouve une station par son nom. Tolère des erreurs de cases, et d'accents.
  SharedConstStation getStationAware(const std::wstring &name) const;

  SharedConstStation getStation(const int id) const;

  void clear();

  std::size_t size() const;

  const_iterator begin() const;
  const_iterator end() const;
};


inline std::size_t StationHandler::size() const
{
	return stationVect_.size();
}

inline StationHandler::const_iterator StationHandler::begin() const
{
	return stationVect_.begin();
}

inline StationHandler::const_iterator StationHandler::end() const
{
	return stationVect_.end();
}

#endif // STATIONHANDLER_H_INCLUDED
