/**
 * @file Tester.cpp
 * @author Loïc HAMOT
 * @date 06/2009
 *
 * La distribution et l'usage sous forme de source ou binaire,
 * avec ou sans modification, sont permis sans restriction.
 * En particulier, je donne mon accord au site developpez.com
 * pour l'utilisation de tout ou partie des sources et de la documentation
 * de ce projet dans les pages developpez.com
 */

#include "Tester.h"

#include <ctime>
#include <iostream>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "PathResult.h"
#include "Line.h"
#include "Station.h"
#include "DVPStationDBLoader.h"
#include "IPathFinder.h"

using std::wstring;
using std::wcout;
using std::wcin;
using std::endl;
using std::vector;
using boost::shared_ptr;

#define TEST(assertion) \
{if((assertion) == false) \
{ \
	wcout << #assertion << L"  à echoué" << endl; \
	wstring truc; \
	wcin >> truc; \
} \
else \
wcout << "OK" << endl;}

void Tester::run()
{
	srand(static_cast<unsigned int>(time(0)));

	TEST("Test du TEST(Ce test DOIT échouer)" && 0);

	boost::shared_ptr<DVPStationDBLoader> dbFiller_;//(new DVPStationDBLoader("test1.txt"));
	StationsDB stationDB_;
	StationHandler &stationHandler = stationDB_.getStationHandler();
	LineHandler &lineHandler = stationDB_.getLineHandler();

	wcout << "Chargement fichier empty.txt" << endl;
	dbFiller_.reset(new DVPStationDBLoader("empty.txt"));
	dbFiller_->load(stationDB_);
	wcout << "Test nombre de stations (== 0)" << endl;
	TEST(stationHandler.size() == 0);
	wcout << "Test nombre de lignes (== 0)" << endl;
	TEST(lineHandler.size() == 0);

	wcout << "Chargement fichier no_line.txt" << endl;
  dbFiller_.reset(new DVPStationDBLoader("no_line.txt"));
	bool thereWasException = true;
	try
	{
    dbFiller_->load(stationDB_);
    thereWasException = false;
	}
	catch(std::exception &ex)
	{
	  wcout << "Test exception == IStationDBLoader::LoadError" << endl;
	  TEST(dynamic_cast<IStationDBLoader::LoadError*>(&ex));
	}
	wcout << "Test qu'il y a eu une exception" << endl;
	TEST(thereWasException == true);

	wcout << "Test nombre de stations (== 0)" << endl;
	TEST(stationHandler.size() == 0);
	wcout << "Test nombre de lignes (== 0)" << endl;
	TEST(lineHandler.size() == 0);

	wcout << "Chargement fichier test1.txt" << endl;
	dbFiller_.reset(new DVPStationDBLoader("test1.txt"));
	dbFiller_->load(stationDB_);

	wcout << "Test nombre de stations (== 34)" << endl;
	TEST(stationHandler.size() == 34);
	wcout << "Test nombre de lignes (== 5)" << endl;
	TEST(lineHandler.size() == 5);

	wcout << "Initialisation du Finder" << endl;
	boost::shared_ptr<NoganePathFinder> pathFinder_(new NoganePathFinder());
	pathFinder_->init(stationDB_, 2, 3);

	SharedConstStation startStation;
	SharedConstStation endStation;
	vector<SharedConstStation> stationStepList;
	shared_ptr<PathResult> result;

	wcout << "Test AAA -> AAA" << endl;
	startStation = stationHandler.getStation(L"AAA");
	endStation = stationHandler.getStation(L"AAA");
	result = pathFinder_->find(
						 stationDB_,
						 startStation,
						 stationStepList,
						 endStation,
						 false);
	TEST(result->getTotalMinuteDuration() == 0);
	TEST(result->size() == 1);

	wcout << "Test AAA -> BBB" << endl;
	startStation = stationHandler.getStation(L"AAA");
	endStation = stationHandler.getStation(L"BBB");
	result = pathFinder_->find(
						 stationDB_,
						 startStation,
						 stationStepList,
						 endStation,
						 false);
	TEST(result->getTotalMinuteDuration() == 2);

	wcout << "Test AAA -> KKK" << endl;
	startStation = stationHandler.getStation(L"AAA");
	endStation = stationHandler.getStation(L"KKK");
	result = pathFinder_->find(
						 stationDB_,
						 startStation,
						 stationStepList,
						 endStation,
						 false);
	TEST(result->getTotalMinuteDuration() == 11);

	wcout << "Test AAA -> farFarAway" << endl;
	startStation = stationHandler.getStation(L"AAA");
	endStation = stationHandler.getStation(L"farFarAway");
	result = pathFinder_->find(
						 stationDB_,
						 startStation,
						 stationStepList,
						 endStation,
						 false);
	TEST(result->getTotalMinuteDuration() == 0);
	TEST(result->size() == 0);

	dbFiller_.reset(new DVPStationDBLoader("ListeStationsUnicode.txt"));
	dbFiller_->load(stationDB_);

	size_t nbStation = stationHandler.size();
	for (int i = 0; i < 1000; i++)
	{
    int sec1 = rand() % 10;
		int sec2 = rand() % 10;
		pathFinder_->init(stationDB_, sec1, sec2);

		startStation = stationHandler.getStation(rand() % nbStation);
		endStation = stationHandler.getStation(rand() % nbStation);
		wcout << startStation->getName() << " " << endStation->getName() << " ";
		result = pathFinder_->find(
							 stationDB_,
							 startStation,
							 stationStepList,
							 endStation,
							 false);
		unsigned int duration = result->getTotalMinuteDuration();
		swap(startStation, endStation);
		result = pathFinder_->find(
							 stationDB_,
							 startStation,
							 stationStepList,
							 endStation,
							 false);
		if(duration != result->getTotalMinuteDuration())
		{
			wcout << sec1 << " " << sec2 << endl;
			wcout << duration << " " << result->getTotalMinuteDuration() << endl;
		}
		TEST(duration == result->getTotalMinuteDuration());
	}

	wcout << L"Test des parcours avec étapes" << endl;
	for (unsigned int p = 0; p < 2; p++)
	{
		for (int i = 0; i < 10; i++)
		{
      pathFinder_->init(stationDB_, rand() % 10, rand() % 10);

			startStation = stationHandler.getStation(rand() % nbStation);
			endStation = stationHandler.getStation(rand() % nbStation);
			stationStepList.clear();
			const int nbInterStation = rand() % 20;
			for (int j = 0; j < nbInterStation; j++)
				stationStepList.push_back(stationHandler.getStation(rand() % nbStation));

			wcout << startStation->getName() << " " << endStation->getName() << " ";
			result = pathFinder_->find(
								 stationDB_,
								 startStation,
								 stationStepList,
								 endStation,
								 p != 0);
			unsigned int duration = result->getTotalMinuteDuration();
			swap(startStation, endStation);
			reverse(stationStepList.begin(), stationStepList.end());
			result = pathFinder_->find(
								 stationDB_,
								 startStation,
								 stationStepList,
								 endStation,
								 p != 0);
			wcout << nbInterStation << " - " << result->getTotalMinuteDuration() << " " << duration << endl;
			TEST(duration == result->getTotalMinuteDuration());
		}
		wcout << L"Test des parcours avec optimisations des étapes" << endl;
	}


	wcout << "Tout est OK" << endl;

	exit(0);
}
